/***************************************************************************
 *   Copyright (C) 2007 by ASL-ETH                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef CONTENT_H
#define CONTENT_H
#include <dirent.h>
#include <QDialog>
#include <popup.h>
#include <QScrollArea>
#include <QComboBox>
#include <QImage>
#include <QColor>
#include <QTimer>

#ifdef _CH_
#pragma package <opencv>
#endif
#ifndef _EiC
#include <cv.h>
#include <cxcore.h>
#include <highgui.h>
#include <stdio.h>
#include <ctype.h>
#endif

#include <defs.h>
extern "C" {
    #include <match.h>
}
#include <matrice.h>

#include <cmath>
#include <time.h>
#include <dirent.h>
#include <io_config.h>

#include "makekeys.h"
#include "ransac.h"

#define NBCOLORS 54

class QAction;
class QGroupBox;
class QLabel;
class QLineEdit;
class QMenu;
class QMenuBar;
class QPushButton;
class QTextEdit;

class Dialog : public QDialog
{
     Q_OBJECT

public:
    Dialog();

private slots:
    void clearClicked();
    void receiver();
    void Recognition();
    void open1();
    void open2();
    void about();
    void ReadConfig();
    void createMenu();
    void trainImages();
    void printFile();
    void saveFile();
    void DisplayImage(char *img);
    void Capture();
    void CaptureControl();
    void SaveFrame();
    void setValue(int i);
    void SingleFrameCap();
    void ContFrameCap();
    void Quit();
    void DeleteTemplates();
    void saveFrameStateFunction(IplImage *frame);
    void singContFrameOR(IplImage *frame);
    void timerOR();

private:
    Match *FindMatchesWithHT(int width1, int height1, Keypoint keys1, int width2, int height2, Keypoint keys2, int *nbmatch, IplImage *imN=NULL, IplImage *im0=NULL, int n=0, int nbfeatmodelN=0);
    Match *Tester(int width1, int height1, Keypoint keys1, int width2, int height2, Keypoint keys2, int *pnbmatch, int *pnbfeattot);
    void LoadTemplates(Keypoint * klist, IplImage **imlist, int *pnbtp, long * pnbfeattot);
    Matrice AffineParamApprox(Match * tabmatch, int nbmatch, int maxdim, double * error=NULL, int * nbmatchgood=NULL);
    void DisplayMatches(IplImage* im1, IplImage* im2, Match * tabmatch, int nbmatch, Matrice param, char * fileName, Keypoint k1=NULL, Keypoint k2=NULL, int n=0, int numberPic=0, CvPoint *pcenterList=NULL);
    double Set2RangePi(double angle);
    double Set2Range2Pi(double angle);

    Matrice Affine(Matrice vect, Matrice rot, Matrice transl);
    Matrice Affine(Matrice vect, Matrice affineParam);
    Matrice SimilarityParamApprox(Match * tabmatch, int nbmatch, int maxdim, double * perr, int *pnbmatchgood);
    Matrice Transform(Matrice vect, Matrice param);
    Matrice Similarity(Matrice vect, Matrice similarityParam);

    //menues
    QMenuBar *menuBar;
    QMenu *fileMenu;
    QMenu *viewMenu;
    QMenu *helpMenu;
    //actions
    QAction *exitAction;
    QAction *printAction;
    QAction *saveAction;
    QAction *aboutAction;
    QAction *helpAction;
    QAction *normalSizeAct;
    QAction *fitToWindowAct;
    //standard GUI widgets
    QComboBox *comboBox;
    QTextEdit *bigEditor;
    QPushButton *cancelButton;
    QPushButton *browseButton1;
    QPushButton *browseButton2;
    QPushButton *clearButton;
    QPushButton *runButton1;
    QPushButton *runButton2;
    QPushButton *configButton;
    QPushButton *saveButton;
    QPushButton *helpButton;
    QPushButton *openCamera;
    QPushButton *singleFrame;
    QPushButton *continuousFrame;
    QPushButton *saveFrame;
    QPushButton *deleteTemplates;
    QLabel *imageField1;
    QLabel *imageField2;
    QLabel *textLabel;
    QLabel *simpleRecText;
    QLabel *captureTiming;
    QGroupBox *createFirstExclusiveGroup();
    QGroupBox *createSecondExclusiveGroup();
    QGroupBox *createThirdExclusiveGroup();
    QGroupBox *createFourExclusiveGroup();
    QScrollArea *scrollArea1;
    QScrollArea *scrollArea2;
    QImage test;
    QTimer *timer;

    QRgb* qRgb;
    QPushButton *captureButton;
    int cap;
    QSlider *slider;
    int captureTimer;
    int *newImg;
    int saveFrameState;

#ifdef APP_WIN32
    SYSTEMTIME st;
#else
    tm *st;
#endif
    float clusterVar;
    QLabel *clusterLabel;
    int singleFrameCap, contFrameCap, successMatch;
    int globalSaver;
    //IO
    QString fileNameFull;
    QString fileNameTraining;
    DIR *d2;
    IplImage *tempImg;
    IplImage *dest;

    ///GLOBAL VARS
    /// Distance ratio threshold for matching of two sets of features
    float distance_ratio;
    /// Parameter for Hough transform : division in position X
    int x_div_HT;
    /// Parameter for Hough transform : division in position Y
    int y_div_HT;
    /// Parameter for Hough transform : division in angle
    int theta_div_HT;
    /// Parameter for Hough transform : division in scale
    int scale_div_HT;
    /// Parameter for Hough transform : ratio between two divisions in scale
    float scale_factor;
    /// Number min of match to make an affine transform
    int nb_match_min_AT;
    /// Param for match filtering in position with Affine transform
    float agree_pos_AT;
    /// Number min of match to make a similarity transform
    int nb_match_min_ST;
    /// Param for match filtering in position with Similarity transform
    float agree_pos_ST;
    /// Param for match filtering in orientation with Similarity transform
    float agree_ori_ST;
     /// Param for match filtering in scale with Similarity transform
    float agree_scale_ST;
     /// Display : how to set the threshold to extract border
    int border_threshold;
    /// Keypoint list to set the default template (NOT USED YET)
    Keypoint klistEmpty;
    /// Threshold to determine if we cluster the views (norm 0.05)
    float clustering_threshold;
    /// getting rid of redundant model representations
    int squared_Distance;
    ///Does continuous capturing in this interval (milliseconds)
    int contCaptureTimer;

    IplImage *im, *imcol, *imOrig;
    FILE* fp;
    int count1, nbtp, totalnbviews, nbmatch, nbmatchgood, nbfeatmodel, count, numb;
    char globalFilename[100], fileName0_1[100], fileName0[100], fileName1[100], fileName2[100], buf[100], full_name[50], name[100];
    Keypoint klist[NB_MAX_TEMPLATES], k0, k1, k2;
    IplImage *imlist[NB_MAX_TEMPLATES], *im0, *im1;
    Match *tabmatch;
};
#endif
